@@ -1,5 +1,5 @@
 from collections import Counter
-from typing import AbstractSet, Dict, Iterable, Iterator, List
+from typing import AbstractSet, Iterable, Iterator, List
 
 from .sub import TextSubstitutionParser
 
@@ -27,24 +27,23 @@ class StenoRule:
             return self in instance._flags
 
     # Acceptable string values for lexer flags, as read from JSON.
-    is_special = Flag("SPEC")   # Special rule used internally (in other rules). Only referenced by name.
+    is_reference = Flag("REF")  # Rule used internally in other rules as a reference. Should not be matched directly.
     is_stroke = Flag("STRK")    # Exact match for a single stroke, not part of one. Handled by exact dict lookup.
     is_word = Flag("WORD")      # Exact match for a single word. These rules do not adversely affect lexer performance.
     is_rare = Flag("RARE")      # Rule applies to very few words and could specifically cause false positives.
 
     # Acceptable string values for graph and board element flags, as read from JSON or produced by the analyzer.
-    is_separator = Flag("SEP")  # Stroke separator rule.
     is_inversion = Flag("INV")  # Inversion of steno order. Child rule keys will be out of order with respect to parent.
     is_linked = Flag("LINK")    # Rule that uses keys from two strokes. This complicates stroke delimiting.
     is_unmatched = Flag("BAD")  # Placeholder for keys inside a compound rule that do not belong to another child rule.
 
-    def __init__(self, r_id:str, keys:str, letters:str, info:str, flags:AbstractSet[str], alt="") -> None:
-        self.id = r_id          # Rule ID string. Used as a unique identifier. May be empty if dynamically generated.
+    def __init__(self, keys:str, letters:str, info:str, flags:AbstractSet[str]=frozenset(), r_id="", alt="") -> None:
         self.keys = keys        # Raw string of steno keys that make up the rule.
         self.letters = letters  # Raw English text of the word.
         self.info = info        # Textual description of the rule.
         self._flags = flags     # Set of string flags that apply to the rule.
         self._rulemap = []      # List of child rules mapped to letter positions *in order*.
+        self.id = r_id          # Rule ID string. Used as a unique identifier. May be empty if dynamically generated.
         self.alt = alt          # Alternate text specifically for display in diagrams.
 
     def add_connection(self, child:"StenoRule", start:int, length:int) -> None:
@@ -52,6 +51,35 @@ class StenoRule:
         item = self.Connection(child, start, length)
         self._rulemap.append(item)
 
+    @classmethod
+    def join(cls, rules:Iterable["StenoRule"], info="Compound rule.") -> "StenoRule":
+        """ Join several rules into one. """
+        self = cls("", "", info)
+        offset = 0
+        for r in rules:
+            self.keys += r.keys
+            self.letters += r.letters
+            length = len(r.letters)
+            self.add_connection(r, offset, length)
+            offset += length
+        return self
+
+    @classmethod
+    def unmatched(cls, unmatched_keys:str) -> None:
+        """ Placeholder rule mapping leftover keys to an empty string of letters. """
+        return cls(unmatched_keys, "", unmatched_keys + ": unmatched keys", {cls.is_unmatched})
+
+    def add_unmatched(self, unmatched_keys:str) -> None:
+        """ Add a placeholder child rule mapping leftover keys to an empty string of letters. """
+        if self._rulemap:
+            last_item = self._rulemap[-1]
+            last_child_end = last_item.start + last_item.length
+        else:
+            last_child_end = 0
+        remaining_length = len(self.letters) - last_child_end
+        child = self.unmatched(unmatched_keys)
+        self.add_connection(child, last_child_end, remaining_length)
+
     def __bool__(self) -> bool:
         """ Return True if this rule is compound, meaning it is composed of smaller child rules. """
         return bool(self._rulemap)
@@ -64,105 +92,91 @@ class StenoRule:
         """ The standard string representation of a rule is its keys -> letters mapping. """
         return f"{self.keys} → {self.letters}"
 
-    def verify(self, valid_keys:AbstractSet[str], delimiters:AbstractSet[str]) -> None:
+    def verify(self, valid_rtfcre:Iterable[str], delimiters:Iterable[str]) -> None:
         """ Perform integrity checks on this rule. """
-        key_counter = Counter(self.keys)
-        assert key_counter, f"Rule {self.id} is empty"
-        # All keys must be valid RTFCRE characters.
-        assert (key_counter.keys() <= valid_keys), f"Rule {self.id} has invalid keys"
+        key_set = set(self.keys)
+        assert key_set, f"Rule {self.id} is empty"
+        # All key characters must be valid RTFCRE characters.
+        invalid = key_set.difference(valid_rtfcre)
+        assert not invalid, f"Rule {self.id} has invalid characters: {invalid}"
         if self:
-            # Check that the rulemap positions all fall within the legal bounds (i.e. within the parent's letters)
-            # Make sure the child rules contain all the keys of the parent between them, and no extras.
-            parent_len = len(self.letters)
+            # Check that the rulemap positions all fall within our own letter bounds.
+            # Start positions must be non-negative and increasing monotonic.
+            counter = Counter(self.keys)
+            last_start = 0
             for item in self:
-                assert item.start >= 0
-                assert item.length >= 0
-                assert item.start + item.length <= parent_len
-                keys = item.child.keys
-                key_counter.subtract(keys)
-            mismatched = [k for k in key_counter if key_counter[k] and k not in delimiters]
-            assert not mismatched, f"Rule {self.id} has mismatched keys vs. its child rules: {mismatched}"
-
-    @classmethod
-    def analysis(cls, keys:str, letters:str, info:str) -> "StenoRule":
-        return cls("", keys, letters, info, set())
-
-    @classmethod
-    def unmatched(cls, keys:str) -> "StenoRule":
-        """ Return a placeholder rule mapping leftover keys from a lexer result to an empty string of letters. """
-        return cls("", keys, "", "unmatched keys", {cls.is_unmatched})
+                start = item.start
+                assert start >= last_start
+                last_start = start
+                length = item.length
+                assert length >= 0
+                assert start + length <= len(self.letters)
+                counter.subtract(item.child.keys)
+            # Make sure the child rules contain all of our keys between them with no extras (except delimiters).
+            for k in delimiters:
+                del counter[k]
+            assert not +counter, f"Rule {self.id} has more keys than its child rules: {+counter}"
+            assert not -counter, f"Rule {self.id} has fewer keys than its child rules: {-counter}"
 
 
 class StenoRuleParser:
     """ Converts steno rules from JSON arrays to StenoRule objects.
         In order to recursively resolve references, all rule data should be added before any parsing is done. """
 
-    def __init__(self, sub_parser:TextSubstitutionParser) -> None:
+    def __init__(self, sub_parser:TextSubstitutionParser=None) -> None:
+        if sub_parser is None:
+            sub_parser = TextSubstitutionParser()
         self._sub_parser = sub_parser  # Parser specifically for the pattern field.
         self._rule_data = {}           # Dict of other steno rule data fields from JSON.
+        self._rule_memo = {}           # Memo of finished rules.
 
-    def add_rule_data(self, id:str, data:Iterable) -> None:
+    def add_json_data(self, r_id:str, fields:list) -> None:
         """ Add JSON data for a single rule. The fields, in order, are:
             keys:    RTFCRE formatted string of steno strokes.
             pattern: English text pattern string, consisting of raw letters as well as references to other rules.
             flags:   Optional sequence of flag strings.
-            info:    Optional info string for when the rule is displayed in the GUI. """
+            desc:    Optional description string for when the rule is displayed in the GUI. """
         try:
-            keys, pattern, *optional = data
+            keys, pattern, *optional = fields
         except ValueError as e:
-            raise ValueError(f"Not enough data fields for rule {id}") from e
-        flags = optional.pop(0) if optional else ()
-        info = optional.pop(0) if optional else "No description"
+            raise ValueError(f"Not enough data fields for rule {r_id}") from e
+        flags = frozenset(optional.pop(0) if optional else ())
+        desc = optional.pop(0) if optional else "No description"
         if optional:
-            raise ValueError(f"Too many data fields for rule {id}: extra = {optional}")
+            raise ValueError(f"Too many data fields for rule {r_id}: extra = {optional}")
         alt = ""
         if "(" not in pattern and "|" in pattern:
             pattern, alt = pattern.split("|", 1)
-        self._sub_parser.add_mapping(id, pattern)
-        self._rule_data[id] = [keys, flags, info, alt]
+        self._sub_parser.add_mapping(r_id, pattern)
+        self._rule_data[r_id] = [keys, flags, desc, alt]
 
-    def parse(self) -> List[StenoRule]:
-        """ Parse all saved rule data recursively and return the finished rules in a list. """
-        memo = {}
-        return [self._parse(r_id, memo) for r_id in self._rule_data]
+    def add_json_dict(self, d:dict) -> None:
+        """ Add all rule data from a JSON dict. """
+        if not isinstance(d, dict):
+            raise TypeError('Rules data object is not a dict.')
+        for name, data in d.items():
+            self.add_json_data(name, data)
 
-    def _parse(self, r_id:str, memo:Dict[str, StenoRule]) -> StenoRule:
+    def _parse(self, r_id:str) -> StenoRule:
+        """ Return a rule by ID if finished, else parse it recursively. """
+        memo = self._rule_memo
         if r_id in memo:
             return memo[r_id]
-        keys, flags, info, alt = self._rule_data[r_id]
+        keys, flags, desc, alt = self._rule_data[r_id]
         sub_result = self._sub_parser.parse(r_id)
-        rule = memo[r_id] = StenoRule(r_id, keys, sub_result.text, info, set(flags), alt)
-        for sub in sub_result.subs:
-            child = self._parse(sub.ref, memo)
-            rule.add_connection(child, sub.start, sub.length)
+        connections = [(self._parse(sub.ref), sub.start, sub.length) for sub in sub_result.subs]
+        letters = sub_result.text
+        if connections and letters:
+            # Compound rule info includes the complete mapping of keys to letters.
+            info = f'{keys} → {letters}: {desc}'
+        else:
+            # Base rule info includes only the keys to the left of the description.
+            info = f"{keys}: {desc}"
+        rule = self._rule_memo[r_id] = StenoRule(keys, letters, info, flags, r_id, alt)
+        for c in connections:
+            rule.add_connection(*c)
         return rule
 
-
-class StenoRuleCollection:
-    """ Immutable iterable collection of steno rules. """
-
-    def __init__(self, rules:Iterable[StenoRule]) -> None:
-        self._rules = rules
-
-    def __iter__(self) -> Iterator[StenoRule]:
-        """ Iterate through every rule in order. """
-        return iter(self._rules)
-
-    def verify(self, valid_keys:Iterable[str], delimiters:Iterable[str]) -> None:
-        """ Go through each rule and perform integrity checks. """
-        valid_keys = set(valid_keys)
-        delimiters = set(delimiters)
-        for rule in self:
-            rule.verify(valid_keys, delimiters)
-
-    @classmethod
-    def from_dict(cls, d:Dict[str, Iterable]) -> "StenoRuleCollection":
-        """ Return a rule collection parsed from a standard dict. """
-        if not isinstance(d, dict) or not all([hasattr(v, "__iter__") for v in d.values()]):
-            raise TypeError('Rules data object is not a dict of iterables.')
-        sub_parser = TextSubstitutionParser()
-        parser = StenoRuleParser(sub_parser)
-        for name, data in d.items():
-            parser.add_rule_data(name, data)
-        rules = parser.parse()
-        return cls(rules)
+    def parse(self) -> List[StenoRule]:
+        """ Parse all saved rule data and return the finished rules in a list. """
+        return [self._parse(r_id) for r_id in self._rule_data]
